@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# Copyright 2010-2022 Gentoo Authors
+# Copyright 2010-2025 Gentoo Authors
 # Distributed under the terms of the GNU General Public License v2
 #
 # This is a helper which ebuild processes can use
@@ -8,11 +8,6 @@
 import os
 import signal
 
-# For compatibility with Python < 3.8
-raise_signal = getattr(
-    signal, "raise_signal", lambda signum: os.kill(os.getpid(), signum)
-)
-
 
 # Inherit from KeyboardInterrupt to avoid a traceback from asyncio.
 class SignalInterrupt(KeyboardInterrupt):
@@ -313,4 +308,4 @@ except KeyboardInterrupt as e:
     # Prevent traceback on ^C
     signum = getattr(e, "signum", signal.SIGINT)
     signal.signal(signum, signal.SIG_DFL)
-    raise_signal(signum)
+    signal.raise_signal(signum)
