@@ -1,4 +1,4 @@
-# Copyright 1999-2023 Gentoo Authors
+# Copyright 1999-2025 Gentoo Authors
 # Distributed under the terms of the GNU General Public License v2
 
 import functools
@@ -17,11 +17,6 @@ from portage import os
 from portage.util.futures import asyncio
 from portage.util import apply_secpass_permissions, no_color
 
-portage.proxy.lazyimport.lazyimport(
-    globals(),
-    "portage.package.ebuild.doebuild:_global_pid_phases",
-)
-
 
 class AbstractEbuildProcess(SpawnProcess):
     __slots__ = (
@@ -80,12 +75,6 @@ class AbstractEbuildProcess(SpawnProcess):
             self._async_wait()
             return
 
-        if self.background:
-            # Automatically prevent color codes from showing up in logs,
-            # since we're not displaying to a terminal anyway.
-            self.settings["NOCOLOR"] = "true"
-            self.settings["NO_COLOR"] = "true"
-
         start_ipc_daemon = False
         if self._enable_ipc_daemon:
             self.settings.pop("PORTAGE_EBUILD_EXIT_FILE", None)
