@@ -1,34 +1,32 @@
-# Copyright 1998-2020 Gentoo Authors
+# Copyright 1998-2024 Gentoo Authors
 # Distributed under the terms of the GNU General Public License v2
 
 __all__ = ["dbapi"]
 
+import functools
+import logging
 import re
-
-import portage
-
-portage.proxy.lazyimport.lazyimport(
-    globals(),
-    "portage.dbapi.dep_expand:dep_expand@_dep_expand",
-    "portage.dep:Atom,match_from_list,_match_slot",
-    "portage.output:colorize",
-    "portage.util:cmp_sort_key,writemsg",
-    "portage.versions:catsplit,catpkgsplit,vercmp,_pkg_str",
-)
+import sys
+from typing import Any, Dict, List, Optional, Tuple
+from collections.abc import Sequence
 
 from portage.const import MERGING_IDENTIFIER
 
 from portage import os
 from portage import auxdbkeys
 from portage.eapi import _get_eapi_attrs
-from portage.exception import InvalidData
+from portage.exception import (
+    CorruptionKeyError,
+    InvalidBinaryPackageFormat,
+    InvalidData,
+)
 from portage.localization import _
 from _emerge.Package import Package
 
 
 class dbapi:
-    _category_re = re.compile(r"^\w[-.+\w]*$", re.UNICODE)
-    _categories = None
+    _category_re = re.compile(r"^\w[-.+\w]*$", re.ASCII)
+    _categories: Optional[tuple[str, ...]] = None
     _use_mutable = False
     _known_keys = frozenset(auxdbkeys)
     _pkg_str_aux_keys = ("EAPI", "KEYWORDS", "SLOT", "repository")
@@ -37,12 +35,14 @@ class dbapi:
         pass
 
     @property
-    def categories(self):
+    def categories(self) -> tuple[str, ...]:
         """
         Use self.cp_all() to generate a category list. Mutable instances
         can delete the self._categories attribute in cases when the cached
         categories become invalid and need to be regenerated.
         """
+        from portage.versions import catsplit
+
         if self._categories is not None:
             return self._categories
         self._categories = tuple(sorted({catsplit(x)[0] for x in self.cp_all()}))
@@ -51,11 +51,13 @@ class dbapi:
     def close_caches(self):
         pass
 
-    def cp_list(self, cp, use_cache=1):
+    def cp_list(self, cp: str, use_cache: int = 1) -> Any:
         raise NotImplementedError(self)
 
     @staticmethod
-    def _cmp_cpv(cpv1, cpv2):
+    def _cmp_cpv(cpv1, cpv2) -> int:
+        from portage.versions import vercmp
+
         result = vercmp(cpv1.version, cpv2.version)
         if result == 0 and cpv1.build_time is not None and cpv2.build_time is not None:
             result = (cpv1.build_time > cpv2.build_time) - (
@@ -64,18 +66,20 @@ class dbapi:
         return result
 
     @staticmethod
-    def _cpv_sort_ascending(cpv_list):
+    def _cpv_sort_ascending(cpv_list: Sequence[Any]) -> None:
         """
         Use this to sort self.cp_list() results in ascending
         order. It sorts in place and returns None.
         """
+        from portage.util import cmp_sort_key
+
         if len(cpv_list) > 1:
             # If the cpv includes explicit -r0, it has to be preserved
             # for consistency in findname and aux_get calls, so use a
             # dict to map strings back to their original values.
             cpv_list.sort(key=cmp_sort_key(dbapi._cmp_cpv))
 
-    def cpv_all(self):
+    def cpv_all(self) -> list[str]:
         """Return all CPVs in the db
         Args:
                 None
@@ -92,16 +96,18 @@ class dbapi:
             cpv_list.extend(self.cp_list(cp))
         return cpv_list
 
-    def cp_all(self, sort=False):
+    def cp_all(self, sort: bool = False) -> list[str]:
         """Implement this in a child class
         Args
                 sort - return sorted results
         Returns:
                 A list of strings 1 per CP in the datastore
         """
-        return NotImplementedError
+        raise NotImplementedError
 
-    def aux_get(self, mycpv, mylist, myrepo=None):
+    def aux_get(
+        self, mycpv: str, mylist: str, myrepo: Optional[str] = None
+    ) -> list[str]:
         """Return the metadata keys in mylist for mycpv
         Args:
                 mycpv - "sys-apps/foo-1.0"
@@ -113,7 +119,7 @@ class dbapi:
         """
         raise NotImplementedError
 
-    def aux_update(self, cpv, metadata_updates):
+    def aux_update(self, cpv: str, metadata_updates: dict[str, Any]) -> None:
         """
         Args:
           cpv - "sys-apps/foo-1.0"
@@ -123,7 +129,7 @@ class dbapi:
         """
         raise NotImplementedError
 
-    def match(self, origdep, use_cache=1):
+    def match(self, origdep: str, use_cache: int = 1):
         """Given a dependency, try to find packages that match
         Args:
                 origdep - Depend atom
@@ -132,12 +138,16 @@ class dbapi:
         Returns:
                 a list of packages that match origdep
         """
+        from portage.dbapi.dep_expand import dep_expand as _dep_expand
+
         mydep = _dep_expand(origdep, mydb=self, settings=self.settings)
         return list(
             self._iter_match(mydep, self.cp_list(mydep.cp, use_cache=use_cache))
         )
 
-    def _iter_match(self, atom, cpv_iter):
+    def _iter_match(self, atom: str, cpv_iter):
+        from portage.dep import match_from_list
+
         cpv_iter = iter(match_from_list(atom, cpv_iter))
         if atom.repo:
             cpv_iter = self._iter_match_repo(atom, cpv_iter)
@@ -149,11 +159,13 @@ class dbapi:
 
     def _pkg_str(self, cpv, repo):
         """
-        This is used to contruct _pkg_str instances on-demand during
+        This is used to construct _pkg_str instances on-demand during
         matching. If cpv is a _pkg_str instance with slot attribute,
         then simply return it. Otherwise, fetch metadata and construct
         a _pkg_str instance. This may raise KeyError or InvalidData.
         """
+        from portage.versions import _pkg_str
+
         try:
             cpv.slot
         except AttributeError:
@@ -181,6 +193,8 @@ class dbapi:
                     yield pkg_str
 
     def _iter_match_slot(self, atom, cpv_iter):
+        from portage.dep import _match_slot
+
         for cpv in cpv_iter:
             try:
                 pkg_str = self._pkg_str(cpv, atom.repo)
@@ -195,6 +209,7 @@ class dbapi:
         1) Check for required IUSE intersection (need implicit IUSE here).
         2) Check enabled/disabled flag states.
         """
+        from portage.versions import _pkg_str
 
         aux_keys = ["EAPI", "IUSE", "KEYWORDS", "SLOT", "USE", "repository"]
         for cpv in cpv_iter:
@@ -218,6 +233,10 @@ class dbapi:
 
             yield cpv
 
+    @staticmethod
+    def _iuse_implicit_built(iuse_implicit_match, use, flag):
+        return iuse_implicit_match(flag) or flag in use
+
     def _iuse_implicit_cnstr(self, pkg, metadata):
         """
         Construct a callable that checks if a given USE flag should
@@ -252,9 +271,11 @@ class dbapi:
             # This behavior is only used for EAPIs that support IUSE_EFFECTIVE,
             # since built USE settings for earlier EAPIs may contain a large
             # number of irrelevant flags.
-            prof_iuse = iuse_implicit_match
-            enabled = frozenset(metadata["USE"].split()).__contains__
-            iuse_implicit_match = lambda flag: prof_iuse(flag) or enabled(flag)
+            iuse_implicit_match = functools.partial(
+                self._iuse_implicit_built,
+                iuse_implicit_match,
+                frozenset(metadata["USE"].split()),
+            )
 
         return iuse_implicit_match
 
@@ -347,14 +368,17 @@ class dbapi:
         return True
 
     def invalidentry(self, mypath):
+        from portage.output import colorize
+        from portage.util import writemsg
+
         if "/" + MERGING_IDENTIFIER in mypath:
             if os.path.exists(mypath):
                 writemsg(
-                    colorize("BAD", _("INCOMPLETE MERGE:")) + " %s\n" % mypath,
+                    colorize("BAD", _("INCOMPLETE MERGE:")) + f" {mypath}\n",
                     noiselevel=-1,
                 )
         else:
-            writemsg("!!! Invalid db entry: %s\n" % mypath, noiselevel=-1)
+            writemsg(f"!!! Invalid db entry: {mypath}\n", noiselevel=-1)
 
     def update_ents(self, updates, onProgress=None, onUpdate=None):
         """
@@ -368,6 +392,9 @@ class dbapi:
         @type onUpdate: a callable that takes 2 integer arguments:
                 maxval and curval
         """
+        from portage.versions import _pkg_str
+        from portage.update import update_dbentries
+
         cpv_all = self.cpv_all()
         cpv_all.sort()
         maxval = len(cpv_all)
@@ -406,11 +433,14 @@ class dbapi:
             if not updates_list:
                 continue
 
-            metadata_updates = portage.update_dbentries(
-                updates_list, metadata, parent=pkg
-            )
+            metadata_updates = update_dbentries(updates_list, metadata, parent=pkg)
             if metadata_updates:
-                aux_update(cpv, metadata_updates)
+                try:
+                    aux_update(cpv, metadata_updates)
+                except (InvalidBinaryPackageFormat, CorruptionKeyError) as e:
+                    logging.warning(
+                        f"{e.__class__.__name__}: {e}", exc_info=sys.exc_info()
+                    )
                 if onUpdate:
                     onUpdate(maxval, i + 1)
             if onProgress:
@@ -425,6 +455,8 @@ class dbapi:
         Returns:
                 The number of slotmoves this function did
         """
+        from portage.dep import Atom
+
         atom = mylist[1]
         origslot = mylist[2]
         newslot = mylist[3]
@@ -453,7 +485,11 @@ class dbapi:
                 and mycpv.sub_slot
                 and mycpv.sub_slot not in (mycpv.slot, newslot)
             ):
-                newslot = "{}/{}".format(newslot, mycpv.sub_slot)
+                newslot = f"{newslot}/{mycpv.sub_slot}"
             mydata = {"SLOT": newslot + "\n"}
-            self.aux_update(mycpv, mydata)
+            try:
+                self.aux_update(mycpv, mydata)
+            except CorruptionKeyError as e:
+                logging.warning(f"{e.__class__.__name__}: {e}", exc_info=sys.exc_info())
+                continue
         return moves
